<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Analyzer;

use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\drupalmoduleupgrader\DeriverBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

class FlagHookDeriver extends DeriverBase implements ContainerDeriverInterface {

  /**
   * @var \Drupal\Core\StringTranslation\TranslationInterface
   */
  protected $t;

  /**
   * @var array
   */
  protected $config;

  public function __construct(TranslationInterface $translator, array $config) {
    $this->t = $translator;
    $this->config = $config;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('string_translation'),
      $container->get('config.factory')->get('drupalmoduleupgrader.hooks')->get()
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_definition) {
    $derivatives = [];

    foreach ($this->config as $hook => $info) {
      $variables = ['@hook' => 'hook_' . $hook . '()'];

      $derivative = array_merge($base_definition, $info);
      $derivative['hook'] = $hook;
      $derivative['message'] = $this->t->translate($info['message'], $variables);
      $derivative['description'] = $this->t->translate('Analyzes implementations of @hook.', $variables);
      foreach ($derivative['documentation'] as &$doc) {
        $doc['title'] = $this->t->translate($doc['title'], $variables);
      }

      $derivatives[$hook] = $derivative;
    }

    return $derivatives;
  }

}
