<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Analyzer;

use Drupal\drupalmoduleupgrader\AnalyzerBase;
use Drupal\drupalmoduleupgrader\TargetInterface;

/**
 * @Analyzer(
 *  id = "grep",
 *  description = @Translation("Searches for and replaces commonly-used code that has changed in Drupal 8."),
 *  documentation = {
 *    {
 *      "url" = "https://www.drupal.org/node/2067859",
 *      "title" = @Translation("<code>drupal_get_title()</code> and <code>drupal_set_title()</code> were removed")
 *    },
 *    {
 *      "url" = "https://www.drupal.org/node/2181921",
 *      "title" = @Translation("<code>drupal_cron_run()</code> was moved into the cron service")
 *    },
 *    {
 *      "url" = "https://www.drupal.org/node/2219113",
 *      "title" = @Translation("<code>drupal_json_encode</code> and <code>drupal_json_decode</code> were moved"),
 *    },
 *    {
 *      "url" = "https://www.drupal.org/node/2295037",
 *      "title" = @Translation("<code>drupal_is_cli()</code> replaced by <code>PHP_SAPI</code> constant")
 *    },
 *    {
 *      "url" = "https://www.drupal.org/node/2302541",
 *      "title" = @Translation("<code>drupal_valid_path()</code> was moved into the PathValidator service")
 *    },
 *    {
 *      "url" = "https://www.drupal.org/node/2324935",
 *      "title" = @Translation("The global theme variables have been replaced by an ActiveTheme object")
 *    },
 *    {
 *      "url" = "https://www.drupal.org/node/2328179",
 *      "title" = @Translation("<code>_node_revision_access()</code> replaced by a service")
 *    },
 *    {
 *      "url" = "https://www.drupal.org/node/2173683",
 *      "title" = @Translation("The <code>element_*</code> functions have been replaced by methods of the Element class")
 *    },
 *    {
 *      "url" = "https://www.drupal.org/node/2173787",
 *      "title" = @Translation("<code>format_plural()</code> was moved into the translation service")
 *    },
 *    {
 *      "url" = "https://www.drupal.org/node/2328205",
 *      "title" = @Translation("Certain Taxonomy API functions have moved into the TermStorage class.")
 *    },
 *    {
 *      "url" = "https://www.drupal.org/node/2121003",
 *      "title" = @Translation("Form generation functions have moved into the FormBuilder service.")
 *    }
 *  },
 *  message = @Translation("Many common functions, shared variables, and constants have been renamed."),
 *  level = "warning"
 * )
 */
class Grep extends AnalyzerBase {

  /**
   * {@inheritdoc}
   */
  public function analyze(TargetInterface $target) {
    // It's too odious to try and grep through the entire module for the various
    // targets. So we'll just unconditionally flag the issue.
    return $this->buildIssue($target);
  }

}
