<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter\Functions;

use Drupal\drupalmoduleupgrader\TargetInterface;
use Pharborist\ArrayNode;
use Pharborist\ClassMethodCallNode;
use Pharborist\FalseNode;
use Pharborist\Functions\FunctionCallNode;
use Pharborist\Node;
use Pharborist\NullNode;
use Pharborist\StringNode;

/**
 * @Converter(
 *  id = "_entity_load",
 *  deriver = "Drupal\drupalmoduleupgrader\Plugin\DMU\Converter\Functions\EntityLoadDeriver"
 * )
 */
class EntityLoad extends FunctionCallModifier {

  /**
   * {@inheritdoc}
   */
  public function rewrite(FunctionCallNode $call, TargetInterface $target) {
    $arguments = $call->getArguments();

    if (sizeof($arguments) == 2) {
      $entity_type = clone $arguments[0];
      $entity_id = clone $arguments[1];
    }
    else {
      // The entity_type key is added to the plugin definition by the deriver, if the
      // function being handled is node_load(), user_load(), or similar.
      $entity_type = StringNode::fromValue($this->pluginDefinition['entity_type']);
      $entity_id = clone $arguments[0];
    }

    // entity_load('node', FALSE) => \Drupal::entityManager()->getStorage('node')->loadMultiple()
    // entity_load('node', 30) => \Drupal::entityManager()->getStorage('node')->load(30)
    // entity_load('node', [28, 29, 30]) => \Drupal::entityManager()->getStorage('node')->loadMultiple([28, 29, 30])
    return ClassMethodCallNode::create('\Drupal', 'entityManager')
      ->appendMethodCall('getStorage')
      ->appendArgument($entity_type)
      ->appendMethodCall($entity_id instanceof ArrayNode || $entity_id instanceof FalseNode ? 'loadMultiple' : 'load')
      ->appendArgument($entity_id instanceof FalseNode ? NullNode::create('NULL') : $entity_id);
  }

}
