<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter\Functions;

use Drupal\drupalmoduleupgrader\TranslationAwareDeriver;

/**
 * Builds derivative definitions for the entity_load plugin, allowing it to handle things
 * like node_load(), node_load_multiple(), etc.
 */
class EntityLoadDeriver extends TranslationAwareDeriver {

  protected function getLoad($entity_type, array $base_definition) {
    $function = $entity_type . '_load';
    $variables = [ '@function' => $function ];

    $derivative = $base_definition;
    $derivative['function'] = $function;
    $derivative['entity_type'] = $entity_type;
    $derivative['message'] = $this->translator
      ->translate('`@function` is now `EntityInterface::load()`.', $variables);
    $derivative['description'] = $this->translator
      ->translate('Rewrites calls to @function().', $variables);

    return [ $function => $derivative ];
  }

  protected function getLoadMultiple($entity_type, array $base_definition) {
    $function = $entity_type . '_load_multiple';
    $variables = [ '@function' => $function ];

    $derivative = $base_definition;
    $derivative['function'] = $function;
    $derivative['entity_type'] = $entity_type;
    $derivative['message'] = $this->translator
      ->translate('`@function` is now `EntityInterface::loadMultiple()`.', $variables);
    $derivative['description'] = $this->translator
      ->translate('Rewrites calls to @function().', $variables);

    return [ $function => $derivative ];
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_definition) {
    $derivatives = [];

    foreach (['entity', 'node', 'user', 'comment', 'taxonomy_term'] as $entity_type) {
      $derivatives = array_merge(
        $derivatives,
        $this->getLoad($entity_type, $base_definition),
        $this->getLoadMultiple($entity_type, $base_definition)
      );
    }

    return $derivatives;
  }

}
