<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\drupalmoduleupgrader\ConverterBase;
use Drupal\drupalmoduleupgrader\TargetInterface;
use Drupal\drupalmoduleupgrader\Utility\StringTransformTrait;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @Converter(
 *  id = "hook_block_info",
 *  description = @Translation("Converts Drupal 7's hook_block_info() to plugins."),
 *  hook = "block_info"
 * )
 */
class HookBlockInfo extends ConverterBase implements ContainerFactoryPluginInterface {

  use StringTransformTrait;

  /**
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, LoggerInterface $logger) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory')->get('default')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function convert(TargetInterface $target) {
    try {
      $blocks = $this->executeHook($target, 'block_info');
    }
    catch (\LogicException $e) {
      $this->logger->warning($e->getMessage(), [
        'target' => $target->id(),
        'hook' => $this->pluginDefinition['hook'],
      ]);
      return;
    }

    $indexer = $target->getIndexer('hook');

    foreach ($blocks as $id => $info) {
      // Render the block plugin's shell.
      $render = [
        '#theme' => 'dmu_block',
        '#module' => $target->id(),
        '#class' => $this->toTitleCase($id),
        '#block_id' => $id,
        '#block_label' => $info['info'],
        '#configurable' => $indexer->has('block_configure'),
      ];
      $this->writeClass($target, $this->parse($render));
    }
  }

}
