<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter;

use Drupal\drupalmoduleupgrader\ConverterBase;
use Drupal\drupalmoduleupgrader\TargetInterface;

/**
 * @Converter(
 *  id = "hook_url_outbound_alter",
 *  description = @Translation("Converts hook_url_outbound_alter() to a service."),
 *  hook = "url_outbound_alter"
 * )
 */
class HookURLOutboundAlter extends ConverterBase {

  /**
   * {@inheritdoc}
   */
  public function convert(TargetInterface $target) {
    $indexer = $target->getIndexer('hook');
    if (! $indexer->has('url_outbound_alter')) {
      return;
    }

    $this->writeService($target, 'outbound_path_processor', [
      'class' => 'Drupal\\' . $target->id() . '\\OutboundPathProcessor',
      'tags' => [
        [ 'name' => 'path_processor_outbound' ],
      ],
    ]);

    $render = [
      '#theme' => 'dmu_outbound_path_processor',
      '#module' => $target->id(),
    ];
    $processor = $this->parse($render);
    $indexer
      ->get('url_outbound_alter')
      ->get(0)
      ->cloneAsMethodOf($processor)
      ->setName('processOutbound');
    $this->writeClass($target, $processor);
  }

}
