<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter;

use Drupal\drupalmoduleupgrader\ConverterBase;
use Drupal\drupalmoduleupgrader\TargetInterface;
use Pharborist\Filter;
use Pharborist\Functions\FunctionCallNode;
use Pharborist\LineCommentBlockNode;
use Pharborist\NodeCollection;

/**
 * @Converter(
 *  id = "hook_uninstall",
 *  description = @Translation("Removes variable_del() calls from hook_uninstall()."),
 *  hook = "uninstall"
 * )
 */
class HookUninstall extends ConverterBase {

  /**
   * {@inheritdoc}
   */
  public function convert(TargetInterface $target) {
    $indexer = $target->getIndexer('hook');
    if (! $indexer->has('uninstall')) {
      return;
    }

    $function = $indexer->get('uninstall')->get(0);
    $function
      ->find(Filter::isFunctionCall('variable_del'))
      ->each(function(FunctionCallNode $call) {
        /** @var \Pharborist\StatementNode $statement */
        $statement = $call->getStatement();
        $comment = $statement->toComment();
        $statement->replaceWith($comment);
        LineCommentBlockNode::create('You no longer need to delete module variables during uninstall; this is handled automatically.')->insertBefore($comment);
      });

    $target->getCodeManager()->save($function);
  }

}
