<?php

namespace Drupal\drupalmoduleupgrader\Utility;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\drupalmoduleupgrader\TargetInterface;

class FormConverterFactory {

  /**
   * @var \Drupal\Core\StringTranslation\TranslationInterface
   */
  protected $t;

  /**
   * Configuration for the $form_state parametric rewriter.
   *
   * @var array
   */
  protected $config;

  public function __construct(TranslationInterface $translator, ConfigFactory $config_factory) {
    $this->t = $translator;
    $this->config = $config_factory->get('drupalmoduleupgrader.rewriters')->get('form_state');
  }

  /**
   * Creates a FormConverter for a specific form.
   *
   * @param TargetInterface $target
   *  The module which defines the form.
   * @param string $form_id
   *  The original form ID.
   *
   * @return FormConverter
   *
   * @throws \BadMethodCallException if the target module doesn't define
   * the given form.
   */
  public function get(TargetInterface $target, $form_id) {
    $indexer = $target->getIndexer('function');

    if ($indexer->has($form_id)) {
      return new FormConverter($target, $form_id, $this->config);
    }
    else {
      $message = $this->t->translate('{target} does not define form {form_id}.', [
        'target' => $target->id(),
        'form_id' => $form_id,
      ]);
      throw new \BadMethodCallException($message);
    }
  }

}
