<?php

namespace Drupal\drupalmoduleupgrader\Utility\ParametricRewriter;

use Pharborist\Filter;
use Pharborist\Functions\FunctionCallNode;
use Pharborist\Functions\ParameterNode;
use Pharborist\NodeCollection;
use Pharborist\ObjectMethodCallNode;
use Pharborist\Token;

/**
 * Specialized rewriter for $form_state.
 */
class FormStateRewriter extends PropertyAccessRewriter {

  /**
   * {@inheritdoc}
   */
  public function rewrite(ParameterNode $parameter) {
    parent::rewrite($parameter);
    $function = $parameter->getFunction();
    $form_state = Token::variable('$' . $parameter->getName());

    $set_errors = $function->find(Filter::isFunctionCall('form_set_error', 'form_error'));
    foreach ($set_errors as $set_error) {
      /** @var \Pharborist\Functions\FunctionCallNode $error */
      $arguments = $set_error->getArguments();
      $method = $set_error->getName()->getText() == 'form_set_error' ? 'setErrorByName' : 'setError';

      $rewrite = ObjectMethodCallNode::create(clone $form_state, $method)
        ->appendArgument(clone $arguments[0])
        ->appendArgument(clone $arguments[1]);

      $set_error->replaceWith($rewrite);
    }

    $clear_errors = $function->find(Filter::isFunctionCall('form_clear_error'));
    foreach ($clear_errors as $clear_error) {
      $clear_error->replaceWith(ObjectMethodCallNode::create(clone $form_state, 'clearErrors'));
    }

    $get_errors = $function->find(Filter::isFunctionCall('form_get_errors'));
    foreach ($get_errors as $get_error) {
      $get_error->replaceWith(ObjectMethodCallNode::create(clone $form_state, 'getErrors'));
    }

    $get_errors = $function->find(Filter::isFunctionCall('form_get_error'));
    foreach ($get_errors as $get_error) {
      $rewrite = ObjectMethodCallNode::create(clone $form_state, 'getError')
        ->appendArgument($get_error->getArgumentList()->getItem(0));
      $get_error->replaceWith($rewrite);
    }
  }

}
