<?php

/**
 * @file
 * Contains \Drupal\drupalmoduleupgrader\Utility\ParametricRewriter\ParametricRewriterBase.
 */

namespace Drupal\drupalmoduleupgrader\Utility\ParametricRewriter;

use Pharborist\ArrayLookupNode;
use Pharborist\Filter;
use Pharborist\Functions\ParameterNode;
use Pharborist\Node;
use Pharborist\ObjectMethodCallNode;
use Pharborist\ObjectPropertyNode;
use Pharborist\ScalarNode;
use Pharborist\VariableNode;

/**
 * Base class for parametric rewriters.
 */
class ParametricRewriterBase {

  /**
   * @var array
   */
  protected $config;

  /**
   * Sets the rewriter configuration (i.e., type hint to set, what properties
   * to change, etc.)
   *
   * @param array $config
   *
   * @return $this
   */
  public function setConfig(array $config) {
    $this->config = $config;
    return $this;
  }

  /**
   * Rewrites the function in accordance with the parameter.
   */
  public function rewrite(ParameterNode $parameter) {
    if (isset($this->config['type_hint'])) {
      $parameter->setTypeHint($this->config['type_hint']);
    }
  }

  /**
   * Returns every node in the function where the parameter is being read.
   *
   * @return \Pharborist\NodeCollection
   */
  protected function getReads(ParameterNode $parameter) {
    return $parameter
      ->getFunction()
      ->find(Filter::isInstanceOf('\Pharborist\VariableNode'))
      ->filter(function(VariableNode $variable) use ($parameter) {
        return $variable->getName() == $parameter->getName();
      })
      ->not([ $this, 'isAssigned' ]);
  }

  /**
   * Returns every node in the function where the parameter is being written.
   *
   * @return \Pharborist\NodeCollection
   */
  protected function getWrites(ParameterNode $parameter) {
    return $parameter
      ->getFunction()
      ->find(Filter::isInstanceOf('\Pharborist\VariableNode'))
      ->filter(function(VariableNode $variable) use ($parameter) {
        return $variable->getName() == $parameter->getName();
      })
      ->filter([ $this, 'isAssigned' ]);
  }

  /**
   * Returns if the given node is the left operand of an assignment.
   *
   * @param \Pharborist\Node $node
   *
   * @return boolean
   */
  public function isAssigned(Node $node) {
    /** @var \Pharborist\Operator\AssignNode|NULL $assignment */
    $assignment = $node->closest(Filter::isInstanceOf('\Pharborist\Operator\AssignNode'));
    return ($assignment && $assignment->getLeftOperand() === $node);
  }

}
