<?php

/**
 * @file
 * Contains \Drupal\Tests\drupalmoduleupgrader\Unit\Utility\PropertyAccessRewriterTest.
 */

namespace Drupal\Tests\drupalmoduleupgrader\Unit\Utility;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\drupalmoduleupgrader\Utility\ParametricRewriter\FormStateRewriter;
use Drupal\drupalmoduleupgrader\Utility\ParametricRewriter\PropertyAccessRewriter;
use Drupal\Tests\UnitTestCase;
use Pharborist\Functions\ParameterNode;
use Pharborist\Parser;
use Symfony\Component\Yaml\Yaml;

/**
 * @group DMU
 */
class PropertyAccessRewriterTest extends UnitTestCase {

  private function getConfig($data_type) {
    $configuration = Yaml::parse(file_get_contents(__DIR__ . '/../../../../config/install/drupalmoduleupgrader.rewriters.yml'));
    return $configuration[$data_type];
  }

  public function testFormState() {
    $container = new ContainerBuilder();

    $configBuilder = $this->getMockBuilder('\Drupal\Core\Config\Config');
    $configBuilder->disableOriginalConstructor();
    $config = $configBuilder->getMock();
    $config
      ->expects($this->any())
      ->method('get')
      ->with('form_state')
      ->willReturn($this->getConfig('form_state'));

    $configFactory = $this->getMock('\Drupal\Core\Config\ConfigFactoryInterface');
    $configFactory
      ->expects($this->any())
      ->method('get')
      ->with('drupalmoduleupgrader.rewriters')
      ->willReturn($config);

    $container->set('config.factory', $configFactory);

    $original = <<<'END'
function foo(array $form_state) {
  echo $form_state['always_process'];
  $form_state['always_process'] = FALSE;

  echo $form_state['buttons'];
  $form_state['buttons'] = array();

  echo $form_state['cache'];
  $form_state['cache'] = TRUE;

  echo $form_state['complete_form'];

  echo $form_state['executed'];
  $form_state['executed'] = TRUE;

  echo $form_state['groups'];
  $form_state['groups'] = array();

  echo $form_state['has_file_element'];
  $form_state['has_file_element'] = FALSE;

  echo $form_state['input'];
  $form_state['input'] = array();

  echo $form_state['limit_validation_errors'];
  $form_state['limit_validation_errors'] = TRUE;

  echo $form_state['must_validate'];
  $form_state['must_validate'] = TRUE;

  echo $form_state['process_input'];
  $form_state['process_input'] = TRUE;

  echo $form_state['programmed'];
  $form_state['programmed'] = TRUE;

  echo $form_state['programmed_bypass_access_check'];
  $form_state['programmed_bypass_access_check'] = TRUE;

  echo $form_state['rebuild'];
  $form_state['rebuild'] = TRUE;

  echo $form_state['response'];
  $form_state['response'] = array();

  echo $form_state['storage'];
  $form_state['storage'] = array();

  echo $form_state['submit_handlers'];
  $form_state['submit_handlers'] = array();

  echo $form_state['submitted'];
  $form_state['submitted'] = TRUE;

  echo $form_state['temporary'];
  $form_state['temporary'] = array();

  echo $form_state['triggering_element'];
  $form_state['triggering_element'] = array();

  echo $form_state['validate_handlers'];
  $form_state['validate_handlers'] = array();

  echo $form_state['validation_complete'];
  $form_state['validation_complete'] = TRUE;

  form_set_error('foo', 'Foo is baz and that is not good, brah.');
  form_error($element, 'I blorf at this!');
}
END;
    $expected = <<<'END'
function foo(Drupal\Core\Form\FormStateInterface $form_state) {
  echo $form_state->getAlwaysProcess();
  $form_state->setAlwaysProcess(FALSE);

  echo $form_state->getButtons();
  $form_state->setButtons(array());

  echo $form_state->isCached();
  $form_state->setCached(TRUE);

  echo $form_state->getCompleteForm();

  echo $form_state->isExecuted();
  $form_state->setExecuted(TRUE);

  echo $form_state->getGroups();
  $form_state->setGroups(array());

  echo $form_state->hasFileElement();
  $form_state->setHasFileElement(FALSE);

  echo $form_state->getUserInput();
  $form_state->setUserInput(array());

  echo $form_state->getLimitValidationErrors();
  $form_state->setLimitValidationErrors(TRUE);

  echo $form_state->isValidationEnforced();
  $form_state->setValidationEnforced(TRUE);

  echo $form_state->isProcessingInput();
  $form_state->setProcessInput(TRUE);

  echo $form_state->isProgrammed();
  $form_state->setProgrammed(TRUE);

  echo $form_state->isBypassingProgrammedAccessChecks();
  $form_state->setProgrammedBypassAccessCheck(TRUE);

  echo $form_state->isRebuilding();
  $form_state->setRebuild(TRUE);

  echo $form_state->getResponse();
  $form_state->setResponse(array());

  echo $form_state->getStorage();
  $form_state->setStorage(array());

  echo $form_state->getSubmitHandlers();
  $form_state->setSubmitHandlers(array());

  echo $form_state->isSubmitted();
  $form_state->setSubmitted(TRUE);

  echo $form_state->getTemporary();
  $form_state->setTemporary(array());

  echo $form_state->getTriggeringElement();
  $form_state->setTriggeringElement(array());

  echo $form_state->getValidateHandlers();
  $form_state->setValidateHandlers(array());

  echo $form_state->isValidationComplete();
  $form_state->setValidationComplete(TRUE);

  $form_state->setErrorByName('foo', 'Foo is baz and that is not good, brah.');
  $form_state->setError($element, 'I blorf at this!');
}
END;
    $func = Parser::parseSnippet($original);
    $rewriter = new FormStateRewriter($func->getParameterList()->getItem(0), $container);
    $rewriter->execute();
    $this->assertEquals($expected, $func->getText());
  }

  public function testNode() {
    $original = <<<'END'
function foo($node) {
  echo $node->nid;

  echo $node->sticky;
  $node->sticky = 1;

  echo $node->status;
  $node->status = 1;

  echo $node->promoted;
  $node->promoted = 1;

  echo $node->title;
  $node->title = 'Foobaz';

  echo $node->uid;
  $node->uid = 30;

  echo $node->created;
  $node->created = 86400;

  echo $node->is_new;
  echo $node->type;
}
END;
    $expected = <<<'END'
function foo(Drupal\node\NodeInterface $node) {
  echo $node->id();

  echo $node->isSticky();
  $node->setSticky(1);

  echo $node->isPublished();
  $node->setPublished(1);

  echo $node->isPromoted();
  $node->setPromoted(1);

  echo $node->getTitle();
  $node->setTitle('Foobaz');

  echo $node->getOwnerId();
  $node->setOwnerId(30);

  echo $node->getCreatedTime();
  $node->setCreatedTime(86400);

  echo $node->isNew();
  echo $node->getType();
}
END;
    $func = Parser::parseSnippet($original);
    $rewriter = new PropertyAccessRewriter($func->getParameterList()->getItem(0), $this->getConfig('node'));
    $rewriter->execute();
    $this->assertEquals($expected, $func->getText());
  }

  public function testUser() {
    $original = <<<'END'
function foo($account) {
  echo $account->uid;

  echo $account->name;
  $account->name = 'Albus Dumbledore';

  echo $account->mail;
  $account->mail = 'foo@localhost';

  echo $account->pass;
  $account->pass = 'Nothing to see here!';

  echo $account->login;
  $account->login = 12345;

  echo $account->access;
  $account->access = 12345;

  echo $account->created;
  echo $account->is_new;
}
END;
    $expected = <<<'END'
function foo(Drupal\user\UserInterface $account) {
  echo $account->id();

  echo $account->getUsername();
  $account->setUsername('Albus Dumbledore');

  echo $account->getEmail();
  $account->setEmail('foo@localhost');

  echo $account->getPassword();
  $account->setPassword('Nothing to see here!');

  echo $account->getLastLoginTime();
  $account->setLastLoginTime(12345);

  echo $account->getLastAccessTime();
  $account->setLastAccessTime(12345);

  echo $account->getCreatedTime();
  echo $account->isNew();
}
END;
    $func = Parser::parseSnippet($original);
    $rewriter = new PropertyAccessRewriter($func->getParameterList()->getItem(0), $this->getConfig('user'));
    $rewriter->execute();
    $this->assertEquals($expected, $func->getText());
  }

  public function testTaxonomyTerm() {
    $original = <<<'END'
function foo($term) {
  echo $term->tid;

  echo $term->name;
  $term->name = 'Foobaz';

  echo $term->description;
  $term->description = 'Lorem ipsum dolor sit amet...';

  echo $term->weight;
  $term->weight = -10;

  echo $term->vid;

  echo $term->is_new;
}
END;
    $expected = <<<'END'
function foo(Drupal\taxonomy\TermInterface $term) {
  echo $term->id();

  echo $term->getName();
  $term->setName('Foobaz');

  echo $term->getDescription();
  $term->setDescription('Lorem ipsum dolor sit amet...');

  echo $term->getWeight();
  $term->setWeight(-10);

  echo $term->getVocabularyId();

  echo $term->isNew();
}
END;
    $func = Parser::parseSnippet($original);
    $rewriter = new PropertyAccessRewriter($func->getParameterList()->getItem(0), $this->getConfig('taxonomy_term'));
    $rewriter->execute();
    $this->assertEquals($expected, $func->getText());
  }

}
